"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const prompts_1 = __importDefault(require("prompts"));
const fs_1 = require("fs");
const path_1 = require("path");
const Logger_1 = __importDefault(require("../Logger"));
const Xcode_1 = require("../lib/Xcode");
const Npm_1 = require("../lib/Npm");
const OnCancel_1 = __importDefault(require("../lib/OnCancel"));
const OnPremise_1 = require("../lib/OnPremise");
const detect_indent_1 = __importDefault(require("detect-indent"));
const semver_1 = __importDefault(require("semver"));
const DSYM_INSTRUCTIONS = `To configure your project to upload dSYMs, follow the iOS symbolication guide:

    https://docs.bugsnag.com/platforms/ios/symbolication-guide/

  This will enable you to see full native stacktraces. It can't be done automatically.

`;
const HERMES_INSTRUCTIONS = `You are running a version of React Native that we cannot automatically integrate with due to known issues with the build when Hermes is enabled.

    If you cannot upgrade to a later version of React Native (version 0.68 or above), you can use an older version of this CLI (version 7.20.x or earlier)

  or follow the manual integration instructions in our online docs: https://docs.bugsnag.com/platforms/react-native/react-native/manual-setup/')

`;
const BUGSNAG_CLI_INSTRUCTIONS = `The following tasks have been added to your package.json and can be run after a build to upload source maps to BugSnag:

    bugsnag:create-build              - Creates a new build
    bugsnag:upload-android-ndk        - Uploads Android NDK source maps
    bugsnag:upload-android-proguard   - Uploads Android Proguard source maps
    bugsnag:upload-rn-android         - Uploads React Native Android source maps
    bugsnag:upload-dsym               - Uploads iOS dSYMs
    bugsnag:upload-rn-ios             - Uploads React Native iOS source maps
    bugsnag:upload                    - Runs all of the above tasks

    See https://docs.bugsnag.com/platforms/react-native/react-native/showing-full-stacktraces for details.

`;
function run(projectRoot, urls) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const { bugsnagCliIntegration } = yield (0, prompts_1.default)({
                type: 'confirm',
                name: 'bugsnagCliIntegration',
                message: 'Do you want to install the BugSnag CLI to allow you to upload JavaScript source maps?',
                initial: true
            }, { onCancel: OnCancel_1.default });
            if (bugsnagCliIntegration) {
                yield installBugsnagCliPackage(projectRoot, urls);
                const reactNativeVersion = yield (0, Npm_1.detectInstalledVersion)('react-native', projectRoot);
                if (reactNativeVersion) {
                    if (semver_1.default.lt(reactNativeVersion, '0.68.0')) {
                        yield (0, prompts_1.default)({
                            type: 'text',
                            name: 'hermesInstructions',
                            message: HERMES_INSTRUCTIONS,
                            initial: 'Hit enter to continue …'
                        }, { onCancel: OnCancel_1.default });
                    }
                }
                yield writeToPackageJson((0, path_1.join)(projectRoot, 'package.json'), urls[OnPremise_1.UrlType.UPLOAD], urls[OnPremise_1.UrlType.BUILD]);
                yield (0, prompts_1.default)({
                    type: 'text',
                    name: 'bugsnagCliInstructions',
                    message: BUGSNAG_CLI_INSTRUCTIONS,
                    initial: 'Hit enter to continue …'
                }, { onCancel: OnCancel_1.default });
                const { iosIntegration } = yield (0, prompts_1.default)({
                    type: 'confirm',
                    name: 'iosIntegration',
                    message: 'Do you want to update your Xcode build phase to output JavaScript source maps?',
                    initial: true
                }, { onCancel: OnCancel_1.default });
                if (iosIntegration) {
                    Logger_1.default.info('Modifying the Xcode project');
                    yield (0, Xcode_1.updateXcodeProject)(projectRoot, urls[OnPremise_1.UrlType.UPLOAD], reactNativeVersion, Logger_1.default);
                    yield (0, prompts_1.default)({
                        type: 'text',
                        name: 'dsymUploadInstructions',
                        message: DSYM_INSTRUCTIONS,
                        initial: 'Hit enter to continue …'
                    }, { onCancel: OnCancel_1.default });
                }
            }
            return true;
        }
        catch (e) {
            Logger_1.default.error(e);
            return false;
        }
    });
}
exports.default = run;
function installBugsnagCliPackage(projectRoot, urls) {
    return __awaiter(this, void 0, void 0, function* () {
        const alreadyInstalled = yield (0, Npm_1.detectInstalled)('@bugsnag/cli', projectRoot);
        if (alreadyInstalled) {
            Logger_1.default.warn('@bugsnag/cli is already installed, skipping');
            return;
        }
        Logger_1.default.info('Adding @bugsnag/cli dependency');
        const packageManager = yield (0, Npm_1.guessPackageManager)(projectRoot);
        const { version } = yield (0, prompts_1.default)({
            type: 'text',
            name: 'version',
            message: 'If you want the latest version of @bugsnag/cli hit enter, otherwise type the version you want',
            initial: 'latest'
        }, { onCancel: OnCancel_1.default });
        yield (0, Npm_1.install)(packageManager, '@bugsnag/cli', version, true, projectRoot);
        Logger_1.default.success('@bugsnag/cli dependency is installed');
    });
}
function writeToPackageJson(packageJsonPath, uploadUrl, buildUrl) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const data = yield fs_1.promises.readFile(packageJsonPath, 'utf8');
            const packageJson = JSON.parse(data);
            // Default to two spaces if indent cannot be detected
            const existingIndent = (0, detect_indent_1.default)(data).indent || '  ';
            let createBuildCommand = 'bugsnag-cli create-build';
            let rnAndroidUploadCommand = 'bugsnag-cli upload react-native-android';
            let androidNdkUploadCommand = 'bugsnag-cli upload android-ndk android/';
            let androidProguardUploadCommand = 'bugsnag-cli upload android-proguard android/';
            let rnIosUploadCommand = 'bugsnag-cli upload react-native-ios';
            let dsymUploadCommand = 'bugsnag-cli upload dsym ios/';
            if (uploadUrl) {
                rnAndroidUploadCommand += ` --upload-api-root-url=${uploadUrl}`;
                androidNdkUploadCommand += ` --upload-api-root-url=${uploadUrl}`;
                androidProguardUploadCommand += ` --upload-api-root-url=${uploadUrl}`;
                rnIosUploadCommand += ` --upload-api-root-url=${uploadUrl}`;
                dsymUploadCommand += ` --upload-api-root-url=${uploadUrl}`;
            }
            if (buildUrl) {
                createBuildCommand += ` --build-api-root-url=${buildUrl}`;
            }
            packageJson.scripts = Object.assign(Object.assign({}, packageJson.scripts), { 'bugsnag:create-build': createBuildCommand, 'bugsnag:upload-android-ndk': androidNdkUploadCommand, 'bugsnag:upload-android-proguard': androidProguardUploadCommand, 'bugsnag:upload-rn-android': rnAndroidUploadCommand, 'bugsnag:upload-dsym': dsymUploadCommand, 'bugsnag:upload-rn-ios': rnIosUploadCommand, 'bugsnag:upload': androidNdkUploadCommand + ' && ' + androidProguardUploadCommand + ' && ' + rnAndroidUploadCommand + ' && ' + dsymUploadCommand + ' && ' + rnIosUploadCommand });
            const updatedPackageJson = JSON.stringify(packageJson, null, existingIndent);
            yield fs_1.promises.writeFile(packageJsonPath, updatedPackageJson, 'utf8');
        }
        catch (err) {
            console.error(`Error writing package.json: ${err}`);
        }
    });
}
//# sourceMappingURL=AutomateSymbolicationCommand.js.map