"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.configure = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = require("fs");
const DOCS_LINK = 'https://docs.bugsnag.com/platforms/react-native/react-native/#android';
const APP_END_REGEX = /\n\s*<\/application>/;
const PROPERTY_NAMES = {
    apiKey: 'API_KEY',
    notifyEndpoint: 'ENDPOINT_NOTIFY',
    sessionsEndpoint: 'ENDPOINT_SESSIONS'
};
const HUMAN_READABLE_PROPERTY_NAMES = {
    apiKey: 'API key',
    notifyEndpoint: 'notify endpoint',
    sessionsEndpoint: 'sessions endpoint'
};
function configure(projectRoot, options, logger) {
    return __awaiter(this, void 0, void 0, function* () {
        const manifestPath = path_1.default.join(projectRoot, 'android', 'app', 'src', 'main', 'AndroidManifest.xml');
        try {
            const manifest = yield fs_1.promises.readFile(manifestPath, 'utf8');
            const activityStartMatch = /(\s*)<activity/.exec(manifest);
            const appEndMatch = APP_END_REGEX.exec(manifest);
            if (!activityStartMatch || !appEndMatch) {
                logger.warn(getNoMatchMessage(options));
                return;
            }
            let metadataTags = '';
            const activityStartIndent = activityStartMatch[1];
            for (const [key, value] of Object.entries(options)) {
                const propertyName = PROPERTY_NAMES[key];
                const humanReadableName = HUMAN_READABLE_PROPERTY_NAMES[key];
                if (manifest.includes(`com.bugsnag.android.${propertyName}`)) {
                    const capitalised = humanReadableName[0].toUpperCase() + humanReadableName.slice(1);
                    logger.warn(`${capitalised} is already present, skipping`);
                    continue;
                }
                metadataTags += `${activityStartIndent}<meta-data android:name="com.bugsnag.android.${propertyName}" android:value="${value}" />`;
            }
            if (metadataTags === '') {
                return;
            }
            const updatedManifest = manifest.replace(APP_END_REGEX, metadataTags + appEndMatch);
            yield fs_1.promises.writeFile(manifestPath, updatedManifest, 'utf8');
            logger.success('Updated AndroidManifest.xml');
        }
        catch (e) {
            logger.warn(getUnableToFindProjectMessage(options));
        }
    });
}
exports.configure = configure;
function getUnableToFindProjectMessage(options) {
    return `The Android configuration was not in the expected location and so couldn't be updated automatically.

Add your ${humanReadableOptions(options)} to the AndroidManifest.xml in your project.

See ${DOCS_LINK} for more information`;
}
function getNoMatchMessage(options) {
    return `The project's AndroidManifest.xml couldn't be updated automatically as it was in an unexpected format.

Add your ${humanReadableOptions(options)} to the AndroidManifest.xml in your project.

See ${DOCS_LINK} for more information`;
}
function humanReadableOptions(options) {
    return Object.keys(options)
        .map(key => HUMAN_READABLE_PROPERTY_NAMES[key])
        .join(', ')
        .replace(/,(?=[^,]*$)/, ' and');
}
//# sourceMappingURL=AndroidManifest.js.map