"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.insertAndroid = exports.insertIos = exports.insertJs = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = require("fs");
const util_1 = require("util");
const glob_1 = __importDefault(require("glob"));
const asyncGlob = (0, util_1.promisify)(glob_1.default);
const BUGSNAG_JS_IMPORT_INIT = `import Bugsnag from "@bugsnag/react-native";
Bugsnag.start();`;
const BUGSNAG_COCOA_OBJ_C_IMPORT = '#import <Bugsnag/Bugsnag.h>';
const BUGSNAG_COCOA_OBJ_C_INIT = '[Bugsnag start];';
const COCOA_OBJ_C_APP_LAUNCH_REGEX = /(-\s*\(BOOL\)\s*application:\s*\(UIApplication\s\*\)\s*application\s+didFinishLaunchingWithOptions:\s*\(NSDictionary\s*\*\)launchOptions\s*\{\s*)\S/;
const BUGSNAG_COCOA_SWIFT_IMPORT = 'import Bugsnag';
const BUGSNAG_COCOA_SWIFT_INIT = 'Bugsnag.start()';
const COCOA_SWIFT_APP_LAUNCH_REGEX = /(override func application\s*\(\s*_ application:\s*UIApplication,\s*didFinishLaunchingWithOptions launchOptions:\s*\[UIApplication\.LaunchOptionsKey\s*:\s*Any\]\?\s*=\s*nil\)\s*->\s*Bool\s*\{\s)/;
const BUGSNAG_JAVA_IMPORT = 'import com.bugsnag.android.Bugsnag;';
const BUGSNAG_JAVA_INIT = 'Bugsnag.start(this);';
const JAVA_APP_ON_CREATE_REGEX = /(public void onCreate\s*\(\)\s*\{[^]*super\.onCreate\(\);(\s*))\S/;
const BUGSNAG_KOTLIN_IMPORT = 'import com.bugsnag.android.Bugsnag';
const BUGSNAG_KOTLIN_INIT = 'Bugsnag.start(this)';
const KOTLIN_APP_ON_CREATE_REGEX = /(override fun onCreate\s*\(\)\s*\{[^]*super\.onCreate\(\)(\s*))\S/;
const DOCS_LINK = 'https://docs.bugsnag.com/platforms/react-native/react-native/#basic-configuration';
const FAIL_MSG = (filename) => `Failed to update "${filename}" automatically. The file may not exist or it may be in an unexpected format or location.

Bugsnag must be imported manually. See ${DOCS_LINK} for more information.`;
function insertJs(projectRoot, logger) {
    return __awaiter(this, void 0, void 0, function* () {
        logger.info('Adding Bugsnag to the JS layer');
        const indexJsPath = path_1.default.join(projectRoot, 'index.js');
        try {
            const indexJs = yield fs_1.promises.readFile(indexJsPath, 'utf8');
            if (indexJs.includes(BUGSNAG_JS_IMPORT_INIT)) {
                logger.warn('Bugsnag is already included, skipping');
                return;
            }
            yield fs_1.promises.writeFile(indexJsPath, `${BUGSNAG_JS_IMPORT_INIT}\n\n${indexJs}`, 'utf8');
            logger.success('Done');
        }
        catch (e) {
            logger.error(FAIL_MSG('index.js'));
        }
    });
}
exports.insertJs = insertJs;
function insertIos(projectRoot, logger) {
    return __awaiter(this, void 0, void 0, function* () {
        logger.info('Adding Bugsnag to the iOS layer');
        const iosDir = path_1.default.join(projectRoot, 'ios');
        let appDelegatePath;
        try {
            const xcodeprojDir = (yield fs_1.promises.readdir(iosDir)).find(p => p.endsWith('.xcodeproj'));
            if (!xcodeprojDir) {
                logger.warn(FAIL_MSG('AppDelegate'));
                return;
            }
            const appDelegateDirectory = path_1.default.join(iosDir, xcodeprojDir.replace(/\.xcodeproj$/, ''));
            // handle AppDelegate.m, AppDelegate.mm (RN 0.68+) and AppDelegate.swift (RN 0.77+)
            const appDelegateFile = (yield fs_1.promises.readdir(appDelegateDirectory)).find(p => p.startsWith('AppDelegate.m') || p.startsWith('AppDelegate.swift'));
            if (!appDelegateFile) {
                logger.warn(FAIL_MSG('AppDelegate'));
                return;
            }
            appDelegatePath = path_1.default.join(iosDir, xcodeprojDir.replace(/\.xcodeproj$/, ''), appDelegateFile);
        }
        catch (e) {
            logger.error(FAIL_MSG('AppDelegate'));
            return;
        }
        try {
            const isSwift = path_1.default.extname(appDelegatePath) === '.swift';
            const cocoaImport = isSwift ? BUGSNAG_COCOA_SWIFT_IMPORT : BUGSNAG_COCOA_OBJ_C_IMPORT;
            const cocoaInit = isSwift ? BUGSNAG_COCOA_SWIFT_INIT : BUGSNAG_COCOA_OBJ_C_INIT;
            const cocoaAppLaunchRegex = isSwift ? COCOA_SWIFT_APP_LAUNCH_REGEX : COCOA_OBJ_C_APP_LAUNCH_REGEX;
            const indent = isSwift ? '    ' : '  ';
            const appDelegate = yield fs_1.promises.readFile(appDelegatePath, 'utf8');
            if (appDelegate.includes(cocoaImport) || appDelegate.includes(cocoaInit)) {
                logger.warn('Bugsnag is already included, skipping');
                return;
            }
            const appDelegateWithImport = `${cocoaImport}\n${appDelegate}`;
            const appLaunchRes = cocoaAppLaunchRegex.exec(appDelegateWithImport);
            if (!appLaunchRes) {
                logger.warn(FAIL_MSG(path_1.default.basename(appDelegatePath)));
                return;
            }
            yield fs_1.promises.writeFile(appDelegatePath, appDelegateWithImport.replace(appLaunchRes[1], `${appLaunchRes[1]}${indent}${cocoaInit}\n\n`), 'utf8');
            logger.success('Done');
        }
        catch (e) {
            logger.error(FAIL_MSG(path_1.default.basename(appDelegatePath)));
        }
    });
}
exports.insertIos = insertIos;
function insertAndroid(projectRoot, logger) {
    return __awaiter(this, void 0, void 0, function* () {
        logger.info('Adding Bugsnag to the Android layer');
        let mainApplicationPath;
        try {
            const javaDir = path_1.default.join(projectRoot, 'android', 'app', 'src', 'main', 'java');
            const relativeMainApplicationPathJava = (yield asyncGlob('**/*/MainApplication.java', {
                cwd: javaDir
            }))[0];
            const relativeMainApplicationPathKotlin = (yield asyncGlob('**/*/MainApplication.kt', {
                cwd: javaDir
            }))[0];
            const relativeMainApplicationPath = relativeMainApplicationPathJava || relativeMainApplicationPathKotlin;
            if (!relativeMainApplicationPath) {
                return logger.warn(FAIL_MSG('MainApplication'));
            }
            mainApplicationPath = path_1.default.join(javaDir, relativeMainApplicationPath);
        }
        catch (e) {
            logger.warn(FAIL_MSG('MainApplication'));
            return;
        }
        try {
            const isKotlin = path_1.default.extname(mainApplicationPath) === '.kt';
            const bugsnagImport = isKotlin ? BUGSNAG_KOTLIN_IMPORT : BUGSNAG_JAVA_IMPORT;
            const bugsnagInit = isKotlin ? BUGSNAG_KOTLIN_INIT : BUGSNAG_JAVA_INIT;
            const onCreateRegex = isKotlin ? KOTLIN_APP_ON_CREATE_REGEX : JAVA_APP_ON_CREATE_REGEX;
            const mainApplication = yield fs_1.promises.readFile(mainApplicationPath, 'utf8');
            if (mainApplication.includes(bugsnagImport) || mainApplication.includes(bugsnagInit)) {
                logger.warn('Bugsnag is already included, skipping');
                return;
            }
            const mainApplicationWithImport = mainApplication.replace('import', `${bugsnagImport}\nimport`);
            const onCreateRes = onCreateRegex.exec(mainApplicationWithImport);
            if (!onCreateRes) {
                logger.warn(FAIL_MSG('MainApplication'));
                return;
            }
            yield fs_1.promises.writeFile(mainApplicationPath, mainApplicationWithImport.replace(onCreateRes[1], `${onCreateRes[1]}${bugsnagInit}${onCreateRes[2]}`), 'utf8');
            logger.success('Done');
        }
        catch (e) {
            logger.error(FAIL_MSG('MainApplication'));
        }
    });
}
exports.insertAndroid = insertAndroid;
//# sourceMappingURL=Insert.js.map