"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateXcodeProject = void 0;
const fs_1 = require("fs");
const path_1 = __importDefault(require("path"));
const xcode_1 = __importDefault(require("xcode"));
const semver_1 = __importDefault(require("semver"));
const DOCS_LINK = 'https://docs.bugsnag.com/platforms/react-native/react-native/showing-full-stacktraces/#ios';
const UNLOCATED_PROJ_MSG = `The Xcode project was not in the expected location and so couldn't be updated automatically.

Please see ${DOCS_LINK} for more information`;
const EXTRA_INPUT_FILES = ['"$(SRCROOT)/.xcode.env.local"', '"$(SRCROOT)/.xcode.env"'];
const EXTRA_PACKAGER_ARGS = 'export SOURCE_MAP_PATH=$(pwd)/build/sourcemaps\nif [ ! -d "$SOURCE_MAP_PATH" ]; then\n\tmkdir -p "$SOURCE_MAP_PATH";\nfi\nexport EXTRA_PACKAGER_ARGS="--sourcemap-output $(pwd)/build/sourcemaps/main.jsbundle.map"';
function isErrorWithCode(e) {
    return e instanceof Error && 'code' in e && typeof e.code === 'string';
}
function updateXcodeProject(projectRoot, endpoint, reactNativeVersion, logger) {
    var _a, _b, _c;
    return __awaiter(this, void 0, void 0, function* () {
        const iosDir = path_1.default.join(projectRoot, 'ios');
        const xcodeprojDir = (yield fs_1.promises.readdir(iosDir)).find(p => p.endsWith('.xcodeproj'));
        if (!xcodeprojDir) {
            logger.warn(UNLOCATED_PROJ_MSG);
            return;
        }
        const pbxProjPath = path_1.default.join(iosDir, xcodeprojDir, 'project.pbxproj');
        const proj = xcode_1.default.project(pbxProjPath);
        yield new Promise((resolve, reject) => {
            proj.parse((err) => {
                if (err)
                    return reject(err);
                resolve();
            });
        });
        const buildPhaseMap = ((_c = (_b = (_a = proj === null || proj === void 0 ? void 0 : proj.hash) === null || _a === void 0 ? void 0 : _a.project) === null || _b === void 0 ? void 0 : _b.objects) === null || _c === void 0 ? void 0 : _c.PBXShellScriptBuildPhase) || [];
        logger.info('Ensuring React Native build phase outputs source maps');
        const didUpdate = yield updateBuildReactNativeTask(buildPhaseMap, iosDir, reactNativeVersion, logger);
        if (!didUpdate)
            return;
        yield fs_1.promises.writeFile(pbxProjPath, proj.writeSync(), 'utf8');
        logger.success('Written changes to Xcode project');
    });
}
exports.updateXcodeProject = updateXcodeProject;
function updateBuildReactNativeTask(buildPhaseMap, iosDir, reactNativeVersion, logger) {
    return __awaiter(this, void 0, void 0, function* () {
        let didAnythingUpdate = false;
        let didThisUpdate;
        for (const shellBuildPhaseKey in buildPhaseMap) {
            const phase = buildPhaseMap[shellBuildPhaseKey];
            // The shell script can vary slightly... Vanilla RN projects contain
            //   ../node_modules/react-native/scripts/react-native-xcode.sh
            // and ejected Expo projects contain
            //   `node --print "require('path').dirname(require.resolve('react-native/package.json')) + '/scripts/react-native-xcode.sh'"`
            // so we need a little leniency
            if (typeof phase.shellScript === 'string' && phase.shellScript.includes('/react-native-xcode.sh')) {
                if (reactNativeVersion) {
                    // If we're dealing with RN >= 0.69.0 setup the .xcode.env file to export the source maps to our happy path
                    if (semver_1.default.gte(reactNativeVersion, '0.69.0')) {
                        [phase.inputPaths, didThisUpdate] = addExtraInputFiles(shellBuildPhaseKey, phase.inputPaths, logger);
                        if (didThisUpdate) {
                            didAnythingUpdate = true;
                        }
                    }
                    else {
                        // If we're dealing with RN < 0.69.0 add the extra package arguments to the xcode build phase as use of the .xcode.env file is not supported.
                        [phase.shellScript, didThisUpdate] = addExtraPackagerArgs(shellBuildPhaseKey, phase.shellScript, logger);
                        if (didThisUpdate) {
                            didAnythingUpdate = true;
                        }
                    }
                }
            }
        }
        yield updateXcodeEnv(iosDir, logger);
        return didAnythingUpdate;
    });
}
function addExtraInputFiles(phaseId, existingInputFiles, logger) {
    if (arrayContainsElements(existingInputFiles, EXTRA_INPUT_FILES)) {
        logger.info(`The "Bundle React Native Code and Images" build phase (${phaseId}) already includes the required arguments`);
        return [existingInputFiles, false];
    }
    return [EXTRA_INPUT_FILES.concat(existingInputFiles), true];
}
function arrayContainsElements(mainArray, subArray) {
    return subArray.every(element => mainArray.some(mainElement => mainElement === element));
}
function addExtraPackagerArgs(phaseId, existingShellScript, logger) {
    const parsedExistingShellScript = JSON.parse(existingShellScript);
    if (parsedExistingShellScript.includes(EXTRA_PACKAGER_ARGS)) {
        logger.info(`The "Bundle React Native Code and Images" build phase (${phaseId}) already includes the required arguments`);
        return [existingShellScript, false];
    }
    const scriptLines = parsedExistingShellScript.split('\n');
    return [JSON.stringify([EXTRA_PACKAGER_ARGS].concat(scriptLines).join('\n')), true];
}
function updateXcodeEnv(iosDir, logger) {
    return __awaiter(this, void 0, void 0, function* () {
        const searchString = 'SOURCEMAP_FILE=';
        const envFilePath = path_1.default.join(iosDir, '.xcode.env');
        try {
            yield fs_1.promises.readFile(envFilePath);
            const xcodeEnvData = yield fs_1.promises.readFile(envFilePath, 'utf8');
            if (xcodeEnvData === null || xcodeEnvData === void 0 ? void 0 : xcodeEnvData.includes(searchString)) {
                logger.warn(`The .xcode.env file already contains a section for "${searchString}"`);
                return false;
            }
            else {
                const newData = `${xcodeEnvData}\n\n# React Native Source Map File\nexport SOURCE_MAP_PATH=$(pwd)/build/sourcemaps\nif [ ! -d "$SOURCE_MAP_PATH" ]; then\n\tmkdir -p "$SOURCE_MAP_PATH";\nfi\nexport ${searchString}$(pwd)/build/sourcemaps/main.jsbundle.map`;
                yield fs_1.promises.writeFile(envFilePath, newData, 'utf8');
                return true;
            }
        }
        catch (error) {
            if (isErrorWithCode(error)) {
                if (error.code === 'ENOENT') {
                    logger.info(`Creating a new .xcode.env file at ${envFilePath}`);
                    const newData = `export NODE_BINARY=$(command -v node)\n\n# React Native Source Map File\nexport SOURCE_MAP_PATH=$(pwd)/build/sourcemaps\nif [ ! -d "$SOURCE_MAP_PATH" ]; then\n\tmkdir -p "$SOURCE_MAP_PATH";\nfi\nexport ${searchString}$(pwd)/build/sourcemaps/main.jsbundle.map`;
                    yield fs_1.promises.writeFile(envFilePath, newData, 'utf8');
                    return true;
                }
                else {
                    console.error(`Error updating .xcode.env file: ${error.message}`);
                    return false;
                }
            }
            console.error('Error updating .xcode.env file');
            return false;
        }
    });
}
//# sourceMappingURL=Xcode.js.map